@@ -6,40 +6,32 @@
 # Author: Will Woods <wwoods@redhat.com>
 # Author: Cole Robinson <crobinso@redhat.com>
 #
-# This program is free software; you can redistribute it and/or modify it
-# under the terms of the GNU General Public License as published by the
-# Free Software Foundation; either version 2 of the License, or (at your
-# option) any later version.  See http://www.gnu.org/copyleft/gpl.html for
-# the full text of the license.
+# This work is licensed under the GNU GPLv2 or later.
+# See the COPYING file in the top-level directory.
 
 from __future__ import print_function
 
+import argparse
+import base64
+import datetime
 import errno
+import json
 import locale
 from logging import getLogger, DEBUG, INFO, WARN, StreamHandler, Formatter
-import argparse
 import os
 import re
 import socket
 import sys
 import tempfile
 
-# pylint: disable=import-error
-if sys.version_info[0] >= 3:
-    # pylint: disable=no-name-in-module,redefined-builtin
-    from xmlrpc.client import Fault, ProtocolError
-    from urllib.parse import urlparse
-    basestring = (str, bytes)
-else:
-    from xmlrpclib import Fault, ProtocolError
-    from urlparse import urlparse
-# pylint: enable=import-error
-
 import requests.exceptions
 
 import bugzilla
+from bugzilla._compatimports import Fault, ProtocolError, urlparse
+from bugzilla._util import to_encoding
 
-DEFAULT_BZ = 'https://bugzilla.redhat.com/xmlrpc.cgi'
+
+DEFAULT_BZ = 'https://bugzilla.redhat.com'
 
 format_field_re = re.compile("%{([a-z0-9_]+)(?::([^}]*))?}")
 
@@ -50,30 +42,10 @@ log = getLogger(bugzilla.__name__)
 # Util helpers #
 ################
 
-def _is_unittest():
-    return bool(os.getenv("__BUGZILLA_UNITTEST"))
-
-
 def _is_unittest_debug():
     return bool(os.getenv("__BUGZILLA_UNITTEST_DEBUG"))
 
 
-def to_encoding(ustring):
-    string = ''
-    if isinstance(ustring, basestring):
-        string = ustring
-    elif ustring is not None:
-        string = str(ustring)
-
-    if sys.version_info[0] >= 3:
-        return string
-
-    preferred = locale.getpreferredencoding()
-    if _is_unittest():
-        preferred = "UTF-8"
-    return string.encode(preferred, 'replace')
-
-
 def open_without_clobber(name, *args):
     """
     Try to open the given file with the given mode; if that filename exists,
@@ -89,7 +61,7 @@ def open_without_clobber(name, *args):
             if err.errno == errno.EEXIST:
                 name = "%s.%i" % (orig_name, count)
                 count += 1
-            else:
+            else:  # pragma: no cover
                 raise IOError(err.errno, err.strerror, err.filename)
     fobj = open(name, *args)
     if fd != fobj.fileno():
@@ -97,20 +69,6 @@ def open_without_clobber(name, *args):
     return fobj
 
 
-def get_default_url():
-    """
-    Grab a default URL from bugzillarc [DEFAULT] url=X
-    """
-    from bugzilla.base import _open_bugzillarc
-    cfg = _open_bugzillarc()
-    if cfg:
-        cfgurl = cfg.defaults().get("url", None)
-        if cfgurl is not None:
-            log.debug("bugzillarc: found cli url=%s", cfgurl)
-            return cfgurl
-    return DEFAULT_BZ
-
-
 def setup_logging(debug, verbose):
     handler = StreamHandler(sys.stderr)
     handler.setFormatter(Formatter(
@@ -126,7 +84,7 @@ def setup_logging(debug, verbose):
         log.setLevel(WARN)
 
     if _is_unittest_debug():
-        log.setLevel(DEBUG)
+        log.setLevel(DEBUG)  # pragma: no cover
 
 
 ##################
@@ -137,7 +95,9 @@ def _setup_root_parser():
     epilog = 'Try "bugzilla COMMAND --help" for command-specific help.'
     p = argparse.ArgumentParser(epilog=epilog)
 
-    default_url = get_default_url()
+    default_url = bugzilla.Bugzilla.get_rcfile_default_url()
+    if not default_url:
+        default_url = DEFAULT_BZ
 
     # General bugzilla connection options
     p.add_argument('--bugzilla', default=default_url,
@@ -201,8 +161,11 @@ def _parser_add_output_options(p):
     outg.add_argument('--oneline', action='store_const', dest='output',
             const='oneline',
             help="one line summary of the bug (useful for scripts)")
+    outg.add_argument('--json', action='store_const', dest='output',
+            const='json', help="output contents in json format")
     outg.add_argument('--raw', action='store_const', dest='output',
-            const='raw', help="raw output of the bugzilla contents")
+            const='raw', help="raw output of the bugzilla contents. This "
+            "format is unstable and difficult to parse. Use --json instead.")
     outg.add_argument('--outputformat',
             help="Print output in the form given. "
                  "You can use RPM-style tags that match bug "
@@ -285,10 +248,6 @@ def _parser_add_bz_fields(rootp, command):
         "bugzilla instance has a custom field cf_my_field, do:\n"
         "  --field cf_my_field=VALUE")
 
-    # Used by unit tests, not for end user consumption
-    p.add_argument('--__test-return-result', action="store_true",
-        dest="test_return_result", help=argparse.SUPPRESS)
-
     if not cmd_modify:
         _parser_add_output_options(rootp)
 
@@ -302,6 +261,9 @@ def _setup_action_new_parser(subparsers):
     p = subparsers.add_parser("new", description=description)
 
     _parser_add_bz_fields(p, "new")
+    g = p.add_argument_group("'new' specific options")
+    g.add_argument('--private', action='store_true', default=False,
+        help='Mark new comment as private')
 
 
 def _setup_action_query_parser(subparsers):
@@ -348,10 +310,6 @@ def _setup_action_query_parser(subparsers):
             help=argparse.SUPPRESS)
     p.add_argument('-W', '--status_whiteboard_type',
             help=argparse.SUPPRESS)
-    p.add_argument('-B', '--booleantype',
-            help=argparse.SUPPRESS)
-    p.add_argument('--boolean_query', action="append",
-            help=argparse.SUPPRESS)
     p.add_argument('--fixed_in_type', help=argparse.SUPPRESS)
 
 
@@ -403,7 +361,7 @@ def _setup_action_modify_parser(subparsers):
 def _setup_action_attach_parser(subparsers):
     usage = """
 bugzilla attach --file=FILE --desc=DESC [--type=TYPE] BUGID [BUGID...]
-bugzilla attach --get=ATTACHID --getall=BUGID [...]
+bugzilla attach --get=ATTACHID --getall=BUGID [--ignore-obsolete] [...]
 bugzilla attach --type=TYPE BUGID [BUGID...]"""
     description = "Attach files or download attachments."
     p = subparsers.add_parser("attach", description=description, usage=usage)
@@ -420,6 +378,8 @@ bugzilla attach --type=TYPE BUGID [BUGID...]"""
             default=[], help="Download the attachment with the given ID")
     p.add_argument("--getall", "--get-all", metavar="BUGID", action="append",
             default=[], help="Download all attachments on the given bug")
+    p.add_argument('--ignore-obsolete', action="store_true",
+        help='Do not download attachments marked as obsolete.')
     p.add_argument('-l', '--comment', '--long_desc',
             help="Add comment with attachment")
     p.add_argument('--private', action='store_true', default=False,
@@ -427,16 +387,20 @@ bugzilla attach --type=TYPE BUGID [BUGID...]"""
 
 
 def _setup_action_login_parser(subparsers):
-    usage = 'bugzilla login [username [password]]'
+    usage = 'bugzilla login [--api-key] [username [password]]'
     description = """Log into bugzilla and save a login cookie or token.
 Note: These tokens are short-lived, and future Bugzilla versions will no
 longer support token authentication at all. Please use a
-~/.config/python-bugzilla/bugzillarc file with an API key instead."""
+~/.config/python-bugzilla/bugzillarc file with an API key instead, or
+use 'bugzilla login --api-key' and we will save it for you."""
     p = subparsers.add_parser("login", description=description, usage=usage)
-    p.add_argument("pos_username", nargs="?", help="Optional username",
-            metavar="username")
-    p.add_argument("pos_password", nargs="?", help="Optional password",
-            metavar="password")
+    p.add_argument('--api-key', action='store_true', default=False,
+                   help='Prompt for and save an API key into bugzillarc, '
+                        'rather than prompt for username and password.')
+    p.add_argument("pos_username", nargs="?", help="Optional username ",
+                   metavar="username")
+    p.add_argument("pos_password", nargs="?", help="Optional password ",
+                   metavar="password")
 
 
 def setup_parser():
@@ -522,7 +486,7 @@ def _do_query(bz, opt, parser):
         setattr(opt, optname, val.split(","))
 
     include_fields = None
-    if opt.output == 'raw':
+    if opt.output in ['raw', 'json']:
         # 'raw' always does a getbug() call anyways, so just ask for ID back
         include_fields = ['id']
 
@@ -574,13 +538,11 @@ def _do_query(bz, opt, parser):
         alias=opt.alias or None,
         qa_whiteboard=opt.qa_whiteboard or None,
         devel_whiteboard=opt.devel_whiteboard or None,
-        boolean_query=opt.boolean_query or None,
         bug_severity=opt.severity or None,
         priority=opt.priority or None,
         target_release=opt.target_release or None,
         target_milestone=opt.target_milestone or None,
         emailtype=opt.emailtype or None,
-        booleantype=opt.booleantype or None,
         include_fields=include_fields,
         quicksearch=opt.quicksearch or None,
         savedsearch=opt.savedsearch or None,
@@ -592,10 +554,8 @@ def _do_query(bz, opt, parser):
     built_query.update(q)
     q = built_query
 
-    if not q:
+    if not q:  # pragma: no cover
         parser.error("'query' command requires additional arguments")
-    if opt.test_return_result:
-        return q
     return bz.query(q)
 
 
@@ -613,18 +573,18 @@ def _do_info(bz, opt):
         return ret
 
     productname = (opt.components or opt.component_owners or opt.versions)
-    include_fields = ["name", "id"]
     fastcomponents = (opt.components and not opt.active_components)
+
+    include_fields = ["name", "id"]
+    if opt.components or opt.component_owners:
+        include_fields += ["components.name"]
+        if opt.component_owners:
+            include_fields += ["components.default_assigned_to"]
+        if opt.active_components:
+            include_fields += ["components.is_active"]
+
     if opt.versions:
         include_fields += ["versions"]
-    if opt.component_owners:
-        include_fields += [
-            "components.default_assigned_to",
-            "components.name",
-        ]
-    if (opt.active_components and
-        any(["components" in i for i in include_fields])):
-        include_fields += ["components.is_active"]
 
     bz.refresh_products(names=productname and [productname] or None,
             include_fields=include_fields)
@@ -682,92 +642,126 @@ def _convert_to_outputformat(output):
         fmt += "#%{bug_id} %{status} %{assigned_to} %{component}\t"
         fmt += "[%{target_milestone}] %{flags} %{cve}"
 
-    else:
+    else:  # pragma: no cover
         raise RuntimeError("Unknown output type '%s'" % output)
 
     return fmt
 
 
+def _xmlrpc_converter(obj):
+    if "DateTime" in str(obj.__class__):
+        # xmlrpc DateTime object. Convert to date format that
+        # bugzilla REST API outputs
+        dobj = datetime.datetime.strptime(str(obj), '%Y%m%dT%H:%M:%S')
+        return dobj.isoformat() + "Z"
+    if "Binary" in str(obj.__class__):
+        # xmlrpc Binary object. Convert to base64
+        return base64.b64encode(obj.data).decode("utf-8")
+    raise RuntimeError(
+        "Unexpected JSON conversion class=%s" % obj.__class__)
+
+
+def _format_output_json(buglist):
+    out = {"bugs": [b.get_raw_data() for b in buglist]}
+    s = json.dumps(out, default=_xmlrpc_converter, indent=2, sort_keys=True)
+    print(s)
+
+
+def _format_output_raw(buglist):
+    for b in buglist:
+        print("Bugzilla %s: " % b.bug_id)
+        SKIP_NAMES = ["bugzilla"]
+        for attrname in sorted(b.__dict__):
+            if attrname in SKIP_NAMES:
+                continue
+            if attrname.startswith("_"):
+                continue
+            print(to_encoding(u"ATTRIBUTE[%s]: %s" %
+                              (attrname, b.__dict__[attrname])))
+        print("\n\n")
+
+
+def _bug_field_repl_cb(bz, b, matchobj):
+    # whiteboard and flag allow doing
+    #   %{whiteboard:devel} and %{flag:needinfo}
+    # That's what 'rest' matches
+    (fieldname, rest) = matchobj.groups()
+
+    if fieldname == "whiteboard" and rest:
+        fieldname = rest + "_" + fieldname
+
+    if fieldname == "flag" and rest:
+        val = b.get_flag_status(rest)
+
+    elif fieldname in ["flags", "flags_requestee"]:
+        tmpstr = []
+        for f in getattr(b, "flags", []):
+            requestee = f.get('requestee', "")
+            if fieldname == "flags":
+                requestee = ""
+            if fieldname == "flags_requestee":
+                if requestee == "":
+                    continue
+                tmpstr.append("%s" % requestee)
+            else:
+                tmpstr.append("%s%s%s" %
+                        (f['name'], f['status'], requestee))
+
+        val = ",".join(tmpstr)
+
+    elif fieldname == "cve":
+        cves = []
+        for key in getattr(b, "keywords", []):
+            # grab CVE from keywords and blockers
+            if key.find("Security") == -1:
+                continue
+            for bl in b.blocks:
+                cvebug = bz.getbug(bl)
+                for cb in cvebug.alias:
+                    if (cb.find("CVE") != -1 and
+                        cb.strip() not in cves):
+                        cves.append(cb)
+        val = ",".join(cves)
+
+    elif fieldname == "comments":
+        val = ""
+        for c in getattr(b, "comments", []):
+            val += ("\n* %s - %s:\n%s\n" % (c['time'],
+                     c.get("creator", c.get("author", "")), c['text']))
+
+    elif fieldname == "external_bugs":
+        val = ""
+        for e in getattr(b, "external_bugs", []):
+            url = e["type"]["full_url"].replace("%id%", e["ext_bz_bug_id"])
+            if not val:
+                val += "\n"
+            val += "External bug: %s\n" % url
+
+    elif fieldname == "__unicode__":
+        val = b.__unicode__()
+    else:
+        val = getattr(b, fieldname, "")
+
+    vallist = isinstance(val, list) and val or [val]
+    val = ','.join([to_encoding(v) for v in vallist])
+
+    return val
+
+
 def _format_output(bz, opt, buglist):
-    if opt.output == 'raw':
+    if opt.output in ['raw', 'json']:
         buglist = bz.getbugs([b.bug_id for b in buglist])
-        for b in buglist:
-            print("Bugzilla %s: " % b.bug_id)
-            for attrname in sorted(b.__dict__):
-                print(to_encoding(u"ATTRIBUTE[%s]: %s" %
-                                  (attrname, b.__dict__[attrname])))
-            print("\n\n")
+        if opt.output == 'json':
+            _format_output_json(buglist)
+        if opt.output == 'raw':
+            _format_output_raw(buglist)
         return
 
-    def bug_field(matchobj):
-        # whiteboard and flag allow doing
-        #   %{whiteboard:devel} and %{flag:needinfo}
-        # That's what 'rest' matches
-        (fieldname, rest) = matchobj.groups()
-
-        if fieldname == "whiteboard" and rest:
-            fieldname = rest + "_" + fieldname
-
-        if fieldname == "flag" and rest:
-            val = b.get_flag_status(rest)
-
-        elif fieldname in ["flags", "flags_requestee"]:
-            tmpstr = []
-            for f in getattr(b, "flags", []):
-                requestee = f.get('requestee', "")
-                if fieldname == "flags":
-                    requestee = ""
-                if fieldname == "flags_requestee":
-                    if requestee == "":
-                        continue
-                    tmpstr.append("%s" % requestee)
-                else:
-                    tmpstr.append("%s%s%s" %
-                            (f['name'], f['status'], requestee))
-
-            val = ",".join(tmpstr)
-
-        elif fieldname == "cve":
-            cves = []
-            for key in getattr(b, "keywords", []):
-                # grab CVE from keywords and blockers
-                if key.find("Security") == -1:
-                    continue
-                for bl in b.blocks:
-                    cvebug = bz.getbug(bl)
-                    for cb in cvebug.alias:
-                        if cb.find("CVE") == -1:
-                            continue
-                        if cb.strip() not in cves:
-                            cves.append(cb)
-            val = ",".join(cves)
-
-        elif fieldname == "comments":
-            val = ""
-            for c in getattr(b, "comments", []):
-                val += ("\n* %s - %s:\n%s\n" % (c['time'],
-                         c.get("creator", c.get("author", "")), c['text']))
-
-        elif fieldname == "external_bugs":
-            val = ""
-            for e in getattr(b, "external_bugs", []):
-                url = e["type"]["full_url"].replace("%id%", e["ext_bz_bug_id"])
-                if not val:
-                    val += "\n"
-                val += "External bug: %s\n" % url
-
-        elif fieldname == "__unicode__":
-            val = b.__unicode__()
-        else:
-            val = getattr(b, fieldname, "")
-
-        vallist = isinstance(val, list) and val or [val]
-        val = ','.join([to_encoding(v) for v in vallist])
-
-        return val
-
     for b in buglist:
-        print(format_field_re.sub(bug_field, opt.outputformat))
+        # pylint: disable=cell-var-from-loop
+        def cb(matchobj):
+            return _bug_field_repl_cb(bz, b, matchobj)
+        print(format_field_re.sub(cb, opt.outputformat))
 
 
 def _parse_triset(vallist, checkplus=True, checkminus=True, checkequal=True,
@@ -826,13 +820,11 @@ def _do_new(bz, opt, parser):
         sub_component=opt.sub_component or None,
         alias=opt.alias or None,
         comment_tags=opt.comment_tag or None,
+        comment_private=opt.private or None,
     )
 
     _merge_field_opts(ret, opt, parser)
 
-    if opt.test_return_result:
-        return ret
-
     b = bz.createbug(ret)
     b.refresh()
     return [b]
@@ -934,9 +926,6 @@ def _do_modify(bz, parser, opt):
     if not any([update, wbmap, add_tags, rm_tags]):
         parser.error("'modify' command requires additional arguments")
 
-    if opt.test_return_result:
-        return (update, wbmap, add_tags, rm_tags)
-
     if add_tags or rm_tags:
         ret = bz.update_tags(bugid_list,
             tags_add=add_tags, tags_remove=rm_tags)
@@ -954,30 +943,46 @@ def _do_modify(bz, parser, opt):
     # here. This is a bit weird for traditional bugzilla XMLRPC
     log.debug("Adjusting whiteboard fields one by one")
     for bug in bz.getbugs(bugid_list):
-        for wb, (add_list, rm_list) in wbmap.items():
+        update_kwargs = {}
+        for wbkey, (add_list, rm_list) in wbmap.items():
+            bugval = getattr(bug, wbkey) or ""
             for tag in add_list:
-                newval = getattr(bug, wb) or ""
-                if newval:
-                    newval += " "
-                newval += tag
-                bz.update_bugs([bug.id],
-                               bz.build_update(**{wb: newval}))
+                if bugval:
+                    bugval += " "
+                bugval += tag
 
             for tag in rm_list:
-                newval = (getattr(bug, wb) or "").split()
-                for t in newval[:]:
+                bugsplit = bugval.split()
+                for t in bugsplit[:]:
                     if t == tag:
-                        newval.remove(t)
-                bz.update_bugs([bug.id],
-                               bz.build_update(**{wb: " ".join(newval)}))
+                        bugsplit.remove(t)
+                bugval = " ".join(bugsplit)
+
+            update_kwargs[wbkey] = bugval
+
+        bz.update_bugs([bug.id], bz.build_update(**update_kwargs))
 
 
 def _do_get_attach(bz, opt):
-    for bug in bz.getbugs(opt.getall):
-        opt.get += bug.get_attachment_ids()
+    data = {}
 
-    for attid in set(opt.get):
-        att = bz.openattachment(attid)
+    def _process_attachment_data(_attlist):
+        for _att in _attlist:
+            data[_att["id"]] = _att
+
+    if opt.getall:
+        for attlist in bz.get_attachments(opt.getall, None)["bugs"].values():
+            _process_attachment_data(attlist)
+    if opt.get:
+        _process_attachment_data(
+            bz.get_attachments(None, opt.get)["attachments"].values())
+
+    for attdata in data.values():
+        is_obsolete = attdata.get("is_obsolete", None) == 1
+        if opt.ignore_obsolete and is_obsolete:
+            continue
+
+        att = bz.openattachment_data(attdata)
         outfile = open_without_clobber(att.name, "wb")
         data = att.read(4096)
         while data:
@@ -1046,14 +1051,13 @@ def _make_bz_instance(opt):
         tokenfile = opt.tokenfile or -1
         use_creds = True
 
-    bz = bugzilla.Bugzilla(
+    return bugzilla.Bugzilla(
         url=opt.bugzilla,
         cookiefile=cookiefile,
         tokenfile=tokenfile,
         sslverify=opt.sslverify,
         use_creds=use_creds,
         cert=opt.cert)
-    return bz
 
 
 def _handle_login(opt, action, bz):
@@ -1066,11 +1070,17 @@ def _handle_login(opt, action, bz):
         opt.login or opt.username or opt.password)
     username = getattr(opt, "pos_username", None) or opt.username
     password = getattr(opt, "pos_password", None) or opt.password
+    use_key = getattr(opt, "api_key", False)
 
     try:
-        if do_interactive_login:
-            if bz.url:
-                print("Logging into %s" % urlparse(bz.url)[1])
+        if use_key:
+            bz.interactive_save_api_key()
+        elif do_interactive_login:
+            if bz.api_key:
+                print("You already have an API key configured for %s" % bz.url)
+                print("There is no need to cache a login token. Exiting.")
+                sys.exit(0)
+            print("Logging into %s" % urlparse(bz.url)[1])
             bz.interactive_login(username, password,
                     restrict_login=opt.restrict_login)
     except bugzilla.BugzillaError as e:
@@ -1083,11 +1093,6 @@ def _handle_login(opt, action, bz):
         sys.exit(1)
 
     if is_login_command:
-        msg = "Login successful."
-        if bz.cookiefile or bz.tokenfile:
-            msg = "Login successful, token cache updated."
-
-        print(msg)
         sys.exit(0)
 
 
@@ -1100,9 +1105,6 @@ def _main(unittest_bz_instance):
     log.debug("Launched with command line: %s", " ".join(sys.argv))
     log.debug("Bugzilla module: %s", bugzilla)
 
-    # Connect to bugzilla
-    log.info('Connecting to %s', opt.bugzilla)
-
     if unittest_bz_instance:
         bz = unittest_bz_instance
     else:
@@ -1117,28 +1119,18 @@ def _main(unittest_bz_instance):
     ###########################
 
     if hasattr(opt, "outputformat"):
-        if not opt.outputformat and opt.output not in ['raw', None]:
+        if not opt.outputformat and opt.output not in ['raw', 'json', None]:
             opt.outputformat = _convert_to_outputformat(opt.output)
 
     buglist = []
     if action == 'info':
-        if not (opt.products or
-                opt.components or
-                opt.component_owners or
-                opt.versions):
-            parser.error("'info' command requires additional arguments")
-
         _do_info(bz, opt)
 
     elif action == 'query':
         buglist = _do_query(bz, opt, parser)
-        if opt.test_return_result:
-            return buglist
 
     elif action == 'new':
         buglist = _do_new(bz, opt, parser)
-        if opt.test_return_result:
-            return buglist
 
     elif action == 'attach':
         if opt.get or opt.getall:
@@ -1150,10 +1142,8 @@ def _main(unittest_bz_instance):
             _do_set_attach(bz, opt, parser)
 
     elif action == 'modify':
-        modout = _do_modify(bz, parser, opt)
-        if opt.test_return_result:
-            return modout
-    else:
+        _do_modify(bz, parser, opt)
+    else:  # pragma: no cover
         raise RuntimeError("Unexpected action '%s'" % action)
 
     # If we're doing new/query/modify, output our results
@@ -1168,6 +1158,9 @@ def main(unittest_bz_instance=None):
         except (Exception, KeyboardInterrupt):
             log.debug("", exc_info=True)
             raise
+    except KeyboardInterrupt:
+        print("\nExited at user request.")
+        sys.exit(1)
     except (Fault, bugzilla.BugzillaError) as e:
         print("\nServer error: %s" % str(e))
         sys.exit(3)
@@ -1181,15 +1174,11 @@ def main(unittest_bz_instance=None):
     except (socket.error,
             requests.exceptions.HTTPError,
             requests.exceptions.ConnectionError,
+            requests.exceptions.InvalidURL,
             ProtocolError) as e:
         print("\nConnection lost/failed: %s" % str(e))
         sys.exit(2)
 
 
 def cli():
-    try:
-        main()
-    except KeyboardInterrupt:
-        log.debug("", exc_info=True)
-        print("\nExited at user request.")
-        sys.exit(1)
+    main()
